@@ -1,10 +1,6 @@
-from PySide2.QtWidgets import QApplication
-
 __title__ = "Fields editor"
 __description__ = "A plugin to display fields "
-__long_description__ = QApplication.instance().translate(
-    "fields_editor",
-    """
+__long_description__ = """
 <p>This plugin is used to manage the fields visible from the main "variant view"
 window.</p>
 
@@ -17,9 +13,8 @@ consequence of the variant, impact rating, etc.).</li>
 <li><b>samples:</b> fields concerning the individuals sequenced in the project.</li>
 </ul>
 
-<p>As the number of fields is large, it may be interesting to <em>use the keyword
+<p>As the number of fields is large, it may be interesting to <em>use the 
 search button</em> to quickly find those that are relevant in a study.</p>
-""",
-)
+"""
 __author__ = "Sacha schutz"
 __version__ = "1.0.0"
