@@ -2,9 +2,7 @@ from PySide2.QtWidgets import QApplication
 
 __title__ = "Variant info"
 __description__ = "A plugin to display variant info"
-__long_description__ = QApplication.instance().translate(
-    "variant_info",
-    """
+__long_description__ = """
 <p>This plugin allows you to view all the data attached to the currently selected
 variant in the main Variant view window.
 </p>
@@ -15,7 +13,6 @@ a whole, this plugin targets only one variant.
 written comments added by right-clicking on the variant in the main window.
 </p>
 Data is read-only!
-""",
-)
+"""
 __author__ = "Sacha schutz"
 __version__ = "1.0.0"
