@@ -2,13 +2,8 @@ from PySide2.QtWidgets import QApplication
 
 __title__ = "VQL history"
 __description__ = "A VQL history to store previous vql query"
-__long_description__ = QApplication.instance().translate(
-    "vql_history",
-    """
+__long_description__ = """
 <p>This plugin store all VQL queries previously executed </p>
-
-
-""",
-)
+"""
 __author__ = "Charles Monod-Broca"
 __version__ = "1.0.0"
